@@ -7,50 +7,31 @@ use File::Spec;
 use File::Temp;
 sub dprint;
 
+use Mail::Mbox::MessageParser::MetaInfo;
+use Mail::Mbox::MessageParser::Config;
+
 use Mail::Mbox::MessageParser::Perl;
 use Mail::Mbox::MessageParser::Grep;
 use Mail::Mbox::MessageParser::Cache;
 
-use vars qw( @ISA $VERSION $DEBUG $MAX_TESTCHAR_BUFFER_SIZE %PROGRAMS
-             $FROM_PATTERN $UPDATING_CACHE );
+use vars qw( @ISA $VERSION $DEBUG );
+use vars qw( $CACHE $UPDATING_CACHE );
 
 @ISA = qw(Exporter);
 
-$VERSION = '1.20';
+$VERSION = sprintf "%d.%02d%02d", q/1.30.0/ =~ /(\d+)/g;
 $DEBUG = 0;
 
-$MAX_TESTCHAR_BUFFER_SIZE = 1048576;
-
-%PROGRAMS = (
- 'grep' => '/bin/grep',
- 'tzip' => '/home/david/bin/tzip',
- 'gzip' => '/bin/gzip',
- 'compress' => '/bin/gzip',
- 'bzip' => '/usr/bin/bzip2',
- 'bzip2' => '/usr/bin/bzip2',
-);
-
-# X-From-Line is used by Gnus, and From is used by normal Unix
-# format. Newer versions of Gnus use X-Draft-From
-$FROM_PATTERN = q/(?x)^
-    (X-Draft-From:\s|X-From-Line:\s|
-    From\s
-      # Skip names, months, days
-      (?> [^:]+ )
-      # Match time
-      (?: :\d\d){1,2}
-      # Match time zone (EST), hour shift (+0500), and-or year
-      (?: \s+ (?: [A-Z]{2,3} | [+-]?\d{4} ) ){1,3}
-      # smail compatibility
-      (\sremote\sfrom\s.*)?
-    )/;
-
 #-------------------------------------------------------------------------------
 
-*UPDATING_CACHE = \$Mail::Mbox::MessageParser::Cache::UPDATING_CACHE;
-*SETUP_CACHE = \&Mail::Mbox::MessageParser::Cache::SETUP_CACHE;
-*CLEAR_CACHE = \&Mail::Mbox::MessageParser::Cache::CLEAR_CACHE;
-*WRITE_CACHE = \&Mail::Mbox::MessageParser::Cache::WRITE_CACHE;
+# The class-wide cache, which will be read and written when necessary. i.e.
+# read when an folder reader object is created which uses caching, and
+# written when a different cache is specified, or when the program exits, 
+*CACHE = \$Mail::Mbox::MessageParser::MetaInfo::CACHE;
+
+*UPDATING_CACHE = \$Mail::Mbox::MessageParser::MetaInfo::UPDATING_CACHE;
+*SETUP_CACHE = \&Mail::Mbox::MessageParser::MetaInfo::SETUP_CACHE;
+sub SETUP_CACHE;
 
 #-------------------------------------------------------------------------------
 
@@ -176,6 +157,18 @@ sub new
 
 #-------------------------------------------------------------------------------
 
+sub _init
+{
+  my $self = shift;
+
+  $self->{'email_line_number'} = 0;
+  $self->{'email_offset'} = 0;
+  $self->{'email_length'} = 0;
+  $self->{'email_number'} = 0;
+}
+
+#-------------------------------------------------------------------------------
+
 sub DESTROY
 {
   my $self = shift;
@@ -296,9 +289,9 @@ sub _OPEN_FILE_HANDLE
 
   # It must be a known compressed file type
   return (undef,"Can't decompress $file_name--no decompressor available")
-    unless defined $PROGRAMS{$file_type};
+    unless defined $Mail::Mbox::MessageParser::Config{'programs'}{$file_type};
 
-  my $filter_command = "$PROGRAMS{$file_type} -cd '$file_name' |";
+  my $filter_command = "$Mail::Mbox::MessageParser::Config{'programs'}{$file_type} -cd '$file_name' |";
 
   dprint "Calling \"$filter_command\" to decompress file \"$file_name\".";
 
@@ -362,14 +355,15 @@ sub _GET_FILE_TYPE
 
     last unless defined $readResult && $readResult != 0;
 
-    last if _IS_BINARY(\$test_chars);
+    last if _IS_BINARY_MAILBOX(\$test_chars);
 
-    if(CORE::length($test_chars) > $MAX_TESTCHAR_BUFFER_SIZE)
+    if(CORE::length($test_chars) >
+        $Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'})
     {
       if(index($test_chars,"\n\n") == -1 && index($test_chars,"\r\n\r\n") == -1)
       {
         dprint "Couldn't find end of first paragraph after " .
-          "$MAX_TESTCHAR_BUFFER_SIZE bytes."
+          "$Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'} bytes."
       }
 
       last;
@@ -389,7 +383,7 @@ sub _GET_FILE_TYPE
   return 'unknown' unless defined $readResult && $readResult != 0;
 
 
-  unless (_IS_BINARY(\$test_chars))
+  unless (_IS_BINARY_MAILBOX(\$test_chars))
   {
     return 'mailbox' if _IS_MAILBOX(\$test_chars);
     return 'non-mailbox ascii';
@@ -445,14 +439,15 @@ sub _GET_ENDLINE
 
     last unless defined $readResult && $readResult != 0;
 
-    last if _IS_BINARY(\$test_chars);
+    last if _IS_BINARY_MAILBOX(\$test_chars);
 
-    if(CORE::length($test_chars) > $MAX_TESTCHAR_BUFFER_SIZE)
+    if(CORE::length($test_chars) >
+        $Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'})
     {
       if(index($test_chars,"\n") == -1 && index($test_chars,"\r\n") == -1)
       {
         dprint "Couldn't find end of first line after " .
-          "$MAX_TESTCHAR_BUFFER_SIZE bytes."
+          "$Mail::Mbox::MessageParser::Config{'max_testchar_buffer_size'} bytes."
       }
 
       last;
@@ -471,7 +466,7 @@ sub _GET_ENDLINE
 
   return undef unless defined $readResult && $readResult != 0;
 
-  return undef if _IS_BINARY(\$test_chars);
+  return undef if _IS_BINARY_MAILBOX(\$test_chars);
 
   if(index($test_chars,"\r\n") != -1)
   {
@@ -491,7 +486,7 @@ sub _IS_COMPRESSED_TYPE
   
   local $" = '|';
 
-  my @types = keys %PROGRAMS;
+  my @types = qw( gzip tzip bzip bzip2 compress );
   my $file_type_pattern = "(@types)";
 
   return $file_type =~ /^$file_type_pattern$/;
@@ -532,9 +527,9 @@ sub _DO_WINDOWS_DECOMPRESSION
   my $file_type = shift;
 
   return (undef,"Can't decompress file handle--no decompressor available")
-    unless defined $PROGRAMS{$file_type};
+    unless defined $Mail::Mbox::MessageParser::Config{'programs'}{$file_type};
 
-  my $filter_command = "$PROGRAMS{$file_type} -cd";
+  my $filter_command = "$Mail::Mbox::MessageParser::Config{'programs'}{$file_type} -cd";
 
   my ($temp_file_handle, $temp_file_name) =
     File::Temp::tempfile('mail-mbox-messageparser-XXXXXX', SUFFIX => '.tmp', UNLINK => 1);
@@ -566,9 +561,9 @@ sub _DO_NONWINDOWS_DECOMPRESSION
   my $file_type = shift;
 
   return (undef,"Can't decompress file handle--no decompressor available")
-    unless defined $PROGRAMS{$file_type};
+    unless defined $Mail::Mbox::MessageParser::Config{'programs'}{$file_type};
 
-  my $filter_command = "$PROGRAMS{$file_type} -cd";
+  my $filter_command = "$Mail::Mbox::MessageParser::Config{'programs'}{$file_type} -cd";
 
   dprint "Calling \"$filter_command\" to decompress filehandle";
 
@@ -632,21 +627,20 @@ sub _DO_DECOMPRESSION
 
 #-------------------------------------------------------------------------------
 
-# Simulates -B, which consumes data on a stream.
-sub _IS_BINARY
+# Simulates -B, which consumes data on a stream. We only look at the first
+# 1000 characters because the body may have foreign binary-like characters
+sub _IS_BINARY_MAILBOX
 {
-  my $data_length = CORE::length(${$_[0]});
-  my $bin_length = ${$_[0]} =~ tr/[\t\n\x20-\x7e]//c;
+  my $data_length;
+  $data_length = index(${$_[0]},"\n\n");
+  $data_length = index(${$_[0]},"\r\n\r\n") if $data_length == -1;
+  $data_length = CORE::length(${$_[0]}) if $data_length == -1;
+
+  my $bin_length = substr(${$_[0]},0,$data_length) =~ tr/[\t\n\x20-\x7e]//c;
+
   my $non_bin_length = $data_length - $bin_length;
 
-  if (($non_bin_length / $data_length) > .70)
-  {
-    return 0;
-  }
-  else
-  {
-    return 1;
-  }
+  return (($non_bin_length / $data_length) <= .70);
 }
 
 #-------------------------------------------------------------------------------
@@ -660,7 +654,7 @@ sub _IS_MAILBOX
 {
   my $test_characters = shift;
 
-  if ($$test_characters =~ /$FROM_PATTERN/im &&
+  if ($$test_characters =~ /$Mail::Mbox::MessageParser::Config{'from_pattern'}/im &&
       $$test_characters =~ /^(Received[ :]|Date:|Subject:|X-Status:|Status:|To:)/sm)
   {
     return 1;
@@ -675,7 +669,33 @@ sub _IS_MAILBOX
 
 sub reset
 {
-  die "Derived class must provide an implementation";
+  my $self = shift;
+
+  if (_IS_A_PIPE($self->{'file_handle'}))
+  {
+    dprint "Avoiding seek() on a pipe";
+  }
+  else
+  {
+    seek $self->{'file_handle'}, length($self->{'prologue'}), 0
+  }
+
+  $self->{'end_of_file'} = 0;
+
+  $self->{'email_line_number'} = 0;
+  $self->{'email_offset'} = 0;
+  $self->{'email_length'} = 0;
+  $self->{'email_number'} = 0;
+}
+
+#-------------------------------------------------------------------------------
+
+sub _IS_A_PIPE
+{
+  my $file_handle = shift;
+
+  return (-t $file_handle || -S $file_handle || -p $file_handle ||
+    !-f $file_handle || !(seek $file_handle, 0, 1));
 }
 
 #-------------------------------------------------------------------------------
@@ -788,20 +808,31 @@ sub read_next_email
   {
     dprint "Storing data into cache, length " . $self->{'email_length'};
 
-    my $cache = $Mail::Mbox::MessageParser::Cache::CACHE;
+    my $CACHE = $Mail::Mbox::MessageParser::Cache::CACHE;
 
-    $cache->{$self->{'file_name'}}{'lengths'}[$self->{'email_number'}-1] =
+    $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'length'} =
       $self->{'email_length'};
 
-    $cache->{$self->{'file_name'}}{'line_numbers'}[$self->{'email_number'}-1] =
+    $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'line_number'} =
       $self->{'email_line_number'};
 
-    $cache->{$self->{'file_name'}}{'offsets'}[$self->{'email_number'}-1] =
+    $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'offset'} =
       $self->{'email_offset'};
+    $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}-1]{'validated'} =
+      1;
+
+    $CACHE->{$self->{'file_name'}}{'modified'} = 1;
+
+    if ($self->{'end_of_file'})
+    {
+      $UPDATING_CACHE = 0;
+
+      # Last one is always validated
+      $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}]{'validated'} =
+        1;
+    }
 
-    $Mail::Mbox::MessageParser::Cache::CACHE_MODIFIED = 1;
   }
-
 }
 
 1;
@@ -835,6 +866,8 @@ Mail::Mbox::MessageParser - A fast and simple mbox folder reader
       'enable_grep' => 1,
     } );
 
+  die $folder_reader unless ref $folder_reader;
+
   # Any newlines or such before the start of the first email
   my $prologue = $folder_reader->prologue;
   print $prologue;
