@@ -5,14 +5,17 @@ no strict;
 @ISA = qw( Exporter Mail::Mbox::MessageParser );
 
 use strict;
-use Mail::Mbox::MessageParser;
 use Carp;
 
-use vars qw( $VERSION $DEBUG $GREP_DATA );
+use Mail::Mbox::MessageParser;
+use Mail::Mbox::MessageParser::Config;
 
-$VERSION = '1.04';
+use vars qw( $VERSION $DEBUG );
+use vars qw( $CACHE );
 
-$GREP_DATA = {};
+$VERSION = sprintf "%d.%02d%02d", q/1.60.0/ =~ /(\d+)/g;
+
+*CACHE = \$Mail::Mbox::MessageParser::MetaInfo::CACHE;
 
 *DEBUG = \$Mail::Mbox::MessageParser::DEBUG;
 *dprint = \&Mail::Mbox::MessageParser::dprint;
@@ -22,71 +25,47 @@ sub dprint;
 
 sub new
 {
-  my ($proto, $options) = @_;
+  my ($proto, $self) = @_;
 
-  my $class = ref($proto) || $proto;
-  my $self  = {};
-  bless ($self, $class);
-
-  carp "Need file_name option" unless defined $options->{'file_name'};
-  carp "Need file_handle option" unless defined $options->{'file_handle'};
+  carp "Need file_name option" unless defined $self->{'file_name'};
+  carp "Need file_handle option" unless defined $self->{'file_handle'};
 
   return "GNU grep not installed"
-    unless defined $Mail::Mbox::MessageParser::PROGRAMS{'grep'};
+    unless defined $Mail::Mbox::MessageParser::Config{'programs'}{'grep'};
 
-  $self->{'file_handle'} = undef;
-  $self->{'file_handle'} = $options->{'file_handle'}
-    if exists $options->{'file_handle'};
+  bless ($self, __PACKAGE__);
 
-  $self->{'file_name'} = $options->{'file_name'};
-  $self->{'file_name'} = $options->{'file_name'};
-  $self->{'force_processing'} = $options->{'force_processing'}
-    if exists $options->{'force_processing'};
-
-  $self->reset();
-
-  _READ_GREP_DATA($self->{'file_name'},$self->{'force_processing'})
-    unless defined $GREP_DATA->{$self->{'file_name'}};
-
-  return "Couldn't read grep data"
-    unless defined $GREP_DATA->{$self->{'file_name'}};
+  $self->_init();
 
   return $self;
 }
 
 #-------------------------------------------------------------------------------
 
-sub reset
+sub _init
 {
   my $self = shift;
 
-  seek $self->{'file_handle'}, length($self->{'prologue'}), 0
-    if defined $self->{'file_handle'} && defined $self->{'prologue'};
+  $self->SUPER::_init();
 
-  $self->{'end_of_file'} = 0;
-
-  $self->{'email_line_number'} = 0;
-  $self->{'email_offset'} = 0;
-  $self->{'email_length'} = 0;
-  $self->{'email_number'} = 0;
+  $self->_initialize_cache_entry();
 }
 
 #-------------------------------------------------------------------------------
 
-sub _read_prologue
+sub _initialize_cache_entry
 {
   my $self = shift;
+    
+  my @stat = stat $self->{'file_name'};
+      
+  my $size = $stat[7];
+  my $time_stamp = $stat[9];
 
-  dprint "Reading mailbox prologue using grep";
-
-  my $prologue_length = $GREP_DATA->{$self->{'file_name'}}{'offsets'}[0];
-  my $bytes_read = 0;
-
-  do
-  {
-    $bytes_read += read($self->{'file_handle'}, $self->{'prologue'},
-      $prologue_length-$bytes_read, $bytes_read);
-  } while ($bytes_read != $prologue_length);
+  $CACHE->{$self->{'file_name'}}{'size'} = $size;
+  $CACHE->{$self->{'file_name'}}{'time_stamp'} = $time_stamp;
+  $CACHE->{$self->{'file_name'}}{'emails'} =
+    _READ_GREP_DATA($self->{'file_name'});
 }
 
 #-------------------------------------------------------------------------------
@@ -94,7 +73,6 @@ sub _read_prologue
 sub _READ_GREP_DATA
 {
   my $filename = shift;
-  my $force_processing = shift;
 
   my @lines_and_offsets;
 
@@ -102,15 +80,8 @@ sub _READ_GREP_DATA
 
   {
     my @grep_results;
-    
-    if ($force_processing)
-    {
-      @grep_results = `$Mail::Mbox::MessageParser::PROGRAMS{'grep'} --extended-regexp --line-number --byte-offset --binary-files=text "^(X-Draft-From: .*|X-From-Line: .*|From [^:]+(:[0-9][0-9]){1,2} ([A-Z]{2,3} [0-9]{4}|[0-9]{4} [+-][0-9]{4}|[0-9]{4})( remote from .*)?)\r?\$" "$filename"`;
-    }
-    else
-    {
-      @grep_results = `$Mail::Mbox::MessageParser::PROGRAMS{'grep'} --extended-regexp --line-number --byte-offset "^(X-Draft-From: .*|X-From-Line: .*|From [^:]+(:[0-9][0-9]){1,2} ([A-Z]{2,3} [0-9]{4}|[0-9]{4} [+-][0-9]{4}|[0-9]{4})( remote from .*)?)\r?\$" "$filename"`;
-    }
+
+    @grep_results = `$Mail::Mbox::MessageParser::Config{'programs'}{'grep'} --extended-regexp --line-number --byte-offset --binary-files=text "^(X-Draft-From: .*|X-From-Line: .*|From [^:]+(:[0-9][0-9]){1,2}(  *([A-Z]{2,3}|[+-]?[0-9]{4})){1,3}( remote from .*)?)\r?\$" "$filename"`;
 
     dprint "Read " . scalar(@grep_results) . " lines of grep data";
 
@@ -122,28 +93,50 @@ sub _READ_GREP_DATA
     }
   }
 
+  my @emails;
+
   for(my $match_number = 0; $match_number <= $#lines_and_offsets; $match_number++)
   {
     if ($match_number == $#lines_and_offsets)
     {
-      my $filesize = -s $filename; $GREP_DATA->{$filename}{'lengths'}[$match_number] =
+      my $filesize = -s $filename;
+      $emails[$match_number]{'length'} =
         $filesize - $lines_and_offsets[$match_number]{'byte offset'};
     }
     else
     {
-      $GREP_DATA->{$filename}{'lengths'}[$match_number] =
+      $emails[$match_number]{'length'} =
         $lines_and_offsets[$match_number+1]{'byte offset'} -
         $lines_and_offsets[$match_number]{'byte offset'};
     }
 
-    $GREP_DATA->{$filename}{'line_numbers'}[$match_number] =
+    $emails[$match_number]{'line_number'} =
       $lines_and_offsets[$match_number]{'line number'};
 
-    $GREP_DATA->{$filename}{'offsets'}[$match_number] =
+    $emails[$match_number]{'offset'} =
       $lines_and_offsets[$match_number]{'byte offset'};
 
-    $GREP_DATA->{$filename}{'validated'}[$match_number] = 0;
+    $emails[$match_number]{'validated'} = 0;
   }
+
+  return \@emails;
+}
+
+#-------------------------------------------------------------------------------
+
+sub _read_prologue
+{
+  my $self = shift;
+
+  dprint "Reading mailbox prologue using grep";
+
+  my $prologue_length = $CACHE->{$self->{'file_name'}}{'emails'}[0]{'offset'};
+
+  my $bytes_read = 0;
+  do {
+    $bytes_read += read($self->{'file_handle'}, $self->{'prologue'},
+      $prologue_length-$bytes_read, $bytes_read);
+  } while ($bytes_read != $prologue_length);
 }
 
 #-------------------------------------------------------------------------------
@@ -153,18 +146,18 @@ sub read_next_email
   my $self = shift;
 
   $self->{'email_line_number'} =
-    $GREP_DATA->{$self->{'file_name'}}{'line_numbers'}[$self->{'email_number'}];
+    $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}]{'line_number'};
   $self->{'email_offset'} =
-    $GREP_DATA->{$self->{'file_name'}}{'offsets'}[$self->{'email_number'}];
+    $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}]{'offset'};
 
   my $email = '';
 
   LOOK_FOR_NEXT_EMAIL:
   while ($self->{'email_number'} <=
-      $#{$GREP_DATA->{$self->{'file_name'}}{'lengths'}})
+      $#{$CACHE->{$self->{'file_name'}}{'emails'}})
   {
     $self->{'email_length'} =
-      $GREP_DATA->{$self->{'file_name'}}{'lengths'}[$self->{'email_number'}];
+      $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}]{'length'};
 
     {
       my $bytes_read = length($email);
@@ -175,54 +168,51 @@ sub read_next_email
     }
 
     last LOOK_FOR_NEXT_EMAIL
-      if $GREP_DATA->{$self->{'file_name'}}{'validated'}[$self->{'email_number'}];
+      if $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}]{'validated'};
+
+    my $endline = $self->{'endline'};
 
     # Keep looking if the header we found is part of a "Begin Included
     # Message".
-    my $end_of_string = substr($email, -200);
-    my $endline = $self->{'endline'};
-    if ($end_of_string =~
-        /$endline-----(?: Begin Included Message |Original Message)-----$endline[^\r\n]*(?:$endline)*$/i)
+    my $end_of_string = '';
+    my $backup_amount = 100;
+    do
     {
-      dprint "Incorrect start of email found--adjusting grep data";
+      $backup_amount *= 2;
+      $end_of_string = substr($email, -$backup_amount);
+    } while (index($end_of_string, "$endline$endline") == -1 &&
+      $backup_amount < $self->{'email_length'});
 
-      $GREP_DATA->{$self->{'file_name'}}{'lengths'}[$self->{'email_number'}] +=
-        $GREP_DATA->{$self->{'file_name'}}{'lengths'}[$self->{'email_number'}+1];
+    if ($end_of_string =~
+        /$endline-----(?: Begin Included Message |Original Message)-----$endline[^\r\n]*(?:$endline)*$/i ||
+        $end_of_string =~
+          /$endline--[^\r\n]*${endline}Content-type:[^\r\n]*$endline(?:$endline)+$/i)
+    {
+      dprint "Incorrect start of email found--adjusting cache data";
 
-      my $last_email_index = $#{$GREP_DATA->{$self->{'file_name'}}{'lengths'}};
+      $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}]{'length'} +=
+        $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}+1]{'length'};
+
+      my $last_email_index = $#{$CACHE->{$self->{'file_name'}}{'emails'}};
 
       if($self->{'email_number'}+2 <= $last_email_index)
       {
-        @{$GREP_DATA->{$self->{'file_name'}}{'lengths'}}
+        @{$CACHE->{$self->{'file_name'}}{'emails'}}
           [$self->{'email_number'}+1..$last_email_index-1] =
-            @{$GREP_DATA->{$self->{'file_name'}}{'lengths'}}
-            [$self->{'email_number'}+2..$last_email_index];
-
-        @{$GREP_DATA->{$self->{'file_name'}}{'line_numbers'}}
-          [$self->{'email_number'}+1..$last_email_index-1] =
-            @{$GREP_DATA->{$self->{'file_name'}}{'line_numbers'}}
-            [$self->{'email_number'}+2..$last_email_index];
-
-        @{$GREP_DATA->{$self->{'file_name'}}{'offsets'}}
-          [$self->{'email_number'}+1..$last_email_index-1] =
-            @{$GREP_DATA->{$self->{'file_name'}}{'offsets'}}
+            @{$CACHE->{$self->{'file_name'}}{'emails'}}
             [$self->{'email_number'}+2..$last_email_index];
       }
 
-      pop @{$GREP_DATA->{$self->{'file_name'}}{'lengths'}};
-      pop @{$GREP_DATA->{$self->{'file_name'}}{'line_numbers'}};
-      pop @{$GREP_DATA->{$self->{'file_name'}}{'offsets'}};
+      pop @{$CACHE->{$self->{'file_name'}}{'emails'}};
     }
     else
     {
-      $GREP_DATA->{$self->{'file_name'}}{'validated'}[$self->{'email_number'}] = 1;
+      $CACHE->{$self->{'file_name'}}{'emails'}[$self->{'email_number'}]{'validated'} = 1;
       last LOOK_FOR_NEXT_EMAIL;
     }
   }
 
-  $self->{'end_of_file'} = 1
-    if $self->{'email_number'} == 
-      $#{$GREP_DATA->{$self->{'file_name'}}{'lengths'}};
+  $self->{'end_of_file'} = 1 if eof $self->{'file_handle'};
 
   $self->{'email_number'}++;
 
@@ -245,16 +235,16 @@ Mail::Mbox::MessageParser::Grep - A GNU grep-based mbox folder reader
 
   #!/usr/bin/perl
 
-  use Mail::Mbox::MessageParser::Grep;
+  use Mail::Mbox::MessageParser;
 
   my $filename = 'mail/saved-mail';
   my $filehandle = new FileHandle($filename);
 
   my $folder_reader =
-    new Mail::Mbox::MessageParser::Grep( {
+    new Mail::Mbox::MessageParser( {
       'file_name' => $filename,
       'file_handle' => $filehandle,
-      'file_handle' => $filehandle,
+      'enable_grep' => 1,
     } );
 
   die $folder_reader unless ref $folder_reader;
@@ -273,10 +263,9 @@ Mail::Mbox::MessageParser::Grep - A GNU grep-based mbox folder reader
 =head1 DESCRIPTION
 
 This module implements a GNU grep-based mbox folder reader. It can only be
-used when GNU grep is installed on the system. Users are encouraged to use
-Mail::Mbox::MessageParser instead. The base MessageParser module will
-automatically fall back to another reader implementation if this module can
-not be used.
+used when GNU grep is installed on the system. Users must not instantiate this
+class directly--use Mail::Mbox::MessageParser instead. The base MessageParser
+module will automatically manage the use of grep and non-grep implementations.
 
 =head2 METHODS AND FUNCTIONS
 
@@ -287,12 +276,10 @@ the Mail::Mbox::MessageParser documentation.
 =over 4
 
 =item $ref = new( { 'file_name' => <mailbox file name>,
-                    'file_handle' => <mailbox file handle>,
-                    'force_processing' => <1 or 0>, });
+                    'file_handle' => <mailbox file handle>, });
 
     <file_name> - The full filename of the mailbox
     <file_handle> - An opened file handle for the mailbox
-    <force_processing> - true to force processing of files that look invalid
 
 The constructor for the class takes two parameters. I<file_name> is the
 filename of the mailbox.  The I<file_handle> argument is the opened file
@@ -301,6 +288,8 @@ handle to the mailbox. Both arguments are required.
 Returns a reference to a Mail::Mbox::MessageParser object, or a string
 describing the error.
 
+=back
+
 
 =head1 BUGS
 
